@@ -1,14 +1,13 @@
 class Hiera
   module Backend
     module Eyaml
-
-      VERSION = "2.1.0"
-      DESCRIPTION = "Hiera-eyaml is a backend for Hiera which provides OpenSSL encryption/decryption for Hiera properties"
+      VERSION = '4.3.0'
+      DESCRIPTION = 'Hiera-eyaml is a backend for Hiera which provides OpenSSL encryption/decryption for Hiera properties'
 
       class RecoverableError < StandardError
       end
 
-      def self.subcommand= command
+      def self.subcommand=(command)
         @@subcommand = command
       end
 
@@ -16,16 +15,16 @@ class Hiera
         @@subcommand
       end
 
-      def self.default_encryption_scheme= new_encryption
+      def self.default_encryption_scheme=(new_encryption)
         @@default_encryption_scheme = new_encryption
       end
 
       def self.default_encryption_scheme
-        @@default_encryption_scheme ||= "PKCS7"
+        @@default_encryption_scheme ||= 'PKCS7'
         @@default_encryption_scheme
       end
 
-      def self.verbosity_level= new_verbosity_level
+      def self.verbosity_level=(new_verbosity_level)
         @@debug_level = new_verbosity_level
       end
 
@@ -34,14 +33,13 @@ class Hiera
         @@debug_level
       end
 
-      def self.subcommands= commands
+      def self.subcommands=(commands)
         @@subcommands = commands
       end
 
       def self.subcommands
         @@subcommands
       end
-
     end
   end
 end
